/*! \file Tria.h 
 *  \brief: header file for tria object
 */

#ifndef _TRIA_H_
#define _TRIA_H_

/*Headers:*/
/*{{{1*/
#include "./Element.h"
#include "./TriaHook.h"
#include "./TriaRef.h"
class Parameters;
class Inputs;
class IoModel;
class Node;
class Matice;
class Matpar;

#include "../../include/include.h"
#include "../../shared/Exceptions/exceptions.h"
/*}}}*/

class Tria: public Element,public TriaHook,public TriaRef{

	public:

		int  id;

		Node** nodes; // 3 nodes
		Matice* matice;  // 1 material ice
		Matpar* matpar;  // 1 material parameter
 
		Parameters* parameters; //pointer to solution parameters
		Inputs*  inputs;
		Results*  results; 

		/*Tria constructors, destructors {{{1*/
		Tria();
		Tria(int tria_id,int i, IoModel* iomodel,int nummodels);
		~Tria();
		/*}}}*/
		/*Object virtual functions definitions:{{{1 */
		void  Echo();
		void  DeepEcho();
		int   Id(); 
		int   MyRank();
		void  Marshall(char** pmarshalled_dataset);
		int   MarshallSize();
		void  Demarshall(char** pmarshalled_dataset);
		int   Enum();
		Object* copy();
		/*}}}*/
		/*Update virtual functions resolution: {{{1*/
		void  InputUpdateFromSolution(double* solutiong);
		void  InputUpdateFromVector(double* vector, int name, int type);
		void  InputUpdateFromVector(int* vector, int name, int type);
		void  InputUpdateFromVector(bool* vector, int name, int type);
		void  InputUpdateFromConstant(double constant, int name);
		void  InputUpdateFromConstant(int constant, int name);
		void  InputUpdateFromConstant(bool constant, int name);
		/*}}}*/
		/*Element virtual functions definitions: {{{1*/
		void   ComputeBasalStress(Vec sigma_b);
		void   ComputePressure(Vec p_g);
		void   ComputeStrainRate(Vec eps);
		void   Configure(Elements* elements,Loads* loads,DataSet* nodes,Materials* materials,Parameters* parameters);
		void   SetCurrentConfiguration(Elements* elements,Loads* loads,DataSet* nodes,Materials* materials,Parameters* parameters);
		double CostFunction(void);
		void   CreateKMatrix(Mat Kgg);
		void   CreatePVector(Vec pg);
		void   Du(Vec du_g);
		void   GetBedList(double* bed_list);
		void*  GetMatPar();
		void   GetNodes(void** nodes);
		bool   GetOnBed();
		bool   GetShelf(); 
		void   GetSolutionFromInputs(Vec solution);
		void   GetThicknessList(double* thickness_list);
		void   GetVectorFromInputs(Vec vector,int NameEnum);
		void   Gradj(Vec gradient,int control_type);
		void   GradjB(Vec gradient);
		void   GradjDrag(Vec gradient);
		void   InputAXPY(int YEnum, double scalar, int XEnum);
		void   InputControlConstrain(int control_type,double cm_min, double cm_max);
		void   InputConvergence(int* pconverged,double* eps, int* enums,int num_enums,int* criterionenums,double* criterionvalues,int num_criterionenums);
		void   InputDepthAverageAtBase(int enum_type,int average_enum_type);
		void   InputDuplicate(int original_enum,int new_enum);
		void   InputScale(int enum_type,double scale_factor);
		void   InputToResult(int enum_type,int step,double time);
		void   MaterialUpdateFromTemperature(void){ISSMERROR("not implemented yet");};
		double MassFlux(double* segment);
		void   MaxAbsVx(double* pmaxabsvx, bool process_units);
		void   MaxAbsVy(double* pmaxabsvy, bool process_units);
		void   MaxAbsVz(double* pmaxabsvz, bool process_units);
		void   MaxVel(double* pmaxvel, bool process_units);
		void   MaxVx(double* pmaxvx, bool process_units);
		void   MaxVy(double* pmaxvy, bool process_units);
		void   MaxVz(double* pmaxvz, bool process_units);
		void   MinVel(double* pminvel, bool process_units);
		void   MinVx(double* pminvx, bool process_units);
		void   MinVy(double* pminvy, bool process_units);
		void   MinVz(double* pminvz, bool process_units);
		double Misfit(void);
		void   PatchFill(int* pcount, Patch* patch);
		void   PatchSize(int* pnumrows, int* pnumvertices,int* pnumnodes);
		void   ProcessResultsUnits(void);
		double SurfaceArea(void);
		void   Update(int index, IoModel* iomodel,int analysis_counter,int analysis_type);
		void   UpdateGeometry(void);
		/*}}}*/
		/*Tria specific routines:{{{1*/
		void	  CreateKMatrixBalancedthickness(Mat Kgg);
		void	  CreateKMatrixBalancedthickness2(Mat Kgg);
		void	  CreateKMatrixBalancedvelocities(Mat Kgg);
		void	  CreateKMatrixDiagnosticHoriz(Mat Kgg);
		void	  CreateKMatrixDiagnosticHorizFriction(Mat Kgg);
		void	  CreateKMatrixDiagnosticHutter(Mat Kgg);
		void	  CreateKMatrixDiagnosticSurfaceVert(Mat Kgg);
		void	  CreateKMatrixMelting(Mat Kgg);
		void	  CreateKMatrixPrognostic(Mat Kgg);
		void	  CreateKMatrixPrognostic2(Mat Kgg);
		void	  CreateKMatrixSlope(Mat Kgg);
		void	  CreateKMatrixThermal(Mat Kgg);
		void	  CreatePVectorBalancedthickness(Vec pg);
		void	  CreatePVectorBalancedthickness2(Vec pg);
		void	  CreatePVectorBalancedvelocities(Vec pg);
		void	  CreatePVectorDiagnosticBaseVert(Vec pg);
		void	  CreatePVectorDiagnosticHoriz(Vec pg);
		void	  CreatePVectorDiagnosticHutter(Vec pg);
		void	  CreatePVectorPrognostic(Vec pg);
		void	  CreatePVectorPrognostic2(Vec pg);
		void	  CreatePVectorSlope( Vec pg);
		void	  CreatePVectorThermalSheet( Vec pg);
		void	  CreatePVectorThermalShelf( Vec pg);
		double	  GetArea(void);
		double	  GetAreaCoordinate(double x, double y, int which_one);
		void	  GetB(double* B, double* xyz_list, double* gauss_l1l2l3);
		void	  GetBPrime(double* Bprime, double* xyz_list, double* gauss_l1l2l3);
		void	  GetBPrime_prog(double* Bprime_prog, double* xyz_list, double* gauss_l1l2l3);
		void	  GetB_prog(double* B_prog, double* xyz_list, double* gauss_l1l2l3);
		void	  GetDofList(int* doflist,int* pnumberofdofs);
		void	  GetDofList1(int* doflist);
		void	  GetJacobian(double* J, double* xyz_list,double* gauss_l1l2l3);
		void	  GetJacobianDeterminant2d(double*  Jdet, double* xyz_list,double* gauss_l1l2l3);
		void	  GetJacobianDeterminant3d(double*  Jdet, double* xyz_list,double* gauss_l1l2l3);
		void	  GetJacobianInvert(double*  Jinv, double* xyz_list,double* gauss_l1l2l3);
		void	  GetL(double* L, double* xyz_list, double* gauss_l1l2l3,int numdof);
		void	  GetNodalFunctions(double* l1l2l3, double* gauss_l1l2l3);
		void	  GetNodalFunctionsDerivatives(double* dh1dh3,double* xyz_list, double* gauss_l1l2l3);
		void	  GetNodalFunctionsDerivativesReference(double* dl1dl3,double* gauss_l1l2l3);
		void	  GetParameterDerivativeValue(double* p, double* plist,double* xyz_list, double* gauss_l1l2l3);
		void	  GetParameterValue(double* pp, double* plist, double* gauss_l1l2l3);
		void    GetParameterValue(double* pvalue,Node* node,int enumtype);
		void    GetParameterValue(double* pvalue,Node* node1,Node* node2,double gauss_seg,int enumtype);
		void	  GetSolutionFromInputsDiagnosticHoriz(Vec solution);
		void	  GetSolutionFromInputsAdjointHoriz(Vec solution);
		void	  GetSolutionFromInputsDiagnosticHutter(Vec solution);
		void	  GradjDragStokes(Vec gradient);
		void	  InputUpdateFromSolutionAdjointHoriz( double* solution);
		void	  InputUpdateFromSolutionDiagnosticHoriz( double* solution);
		void	  InputUpdateFromSolutionDiagnosticHutter( double* solution);
		void	  InputUpdateFromSolutionBedSlopeX( double* solution);
		void	  InputUpdateFromSolutionBedSlopeY( double* solution);
		void	  InputUpdateFromSolutionSurfaceSlopeX( double* solution);
		void	  InputUpdateFromSolutionSurfaceSlopeY( double* solution);
		void	  InputUpdateFromSolutionPrognostic( double* solution);
		void	  InputUpdateFromSolutionPrognostic2(double* solution);
		void	  InputUpdateFromSolutionBalancedthickness( double* solution);
		void	  InputUpdateFromSolutionBalancedthickness2( double* solution);
		void	  InputUpdateFromSolutionBalancedvelocities( double* solution);
		bool	  IsInput(int name);
		void	  SetClone(int* minranks);
		void*	  SpawnBeam(int g0, int g1);
		void*	  SpawnSing(int g0);
		void	  SurfaceNormal(double* surface_normal, double xyz_list[3][3]);
		/*}}}*/

};
#endif  /* _TRIA_H */
