/*! \file TriaVertexInput.h 
 *  \brief: header file for triavertexinput object
 */


#ifndef _TRIAVERTEXINPUT_H_
#define _TRIAVERTEXINPUT_H_

/*Headers:*/
/*{{{1*/
#include "./Input.h"
#include "../Elements/TriaRef.h"
/*}}}*/

class TriaVertexInput: public Input,public TriaRef{

	public:
		/*just hold 3 values for 3 vertices: */
		int    enum_type;
		double values[3];

		/*TriaVertexInput constructors, destructors: {{{1*/
		TriaVertexInput();
		TriaVertexInput(int enum_type,double* values);
		~TriaVertexInput();
		/*}}}*/
		/*Object virtual functions definitions:{{{1 */
		void  Echo();
		void  DeepEcho();
		int   Id(); 
		int   MyRank();
		void  Marshall(char** pmarshalled_dataset);
		int   MarshallSize();
		void  Demarshall(char** pmarshalled_dataset);
		int   Enum();
		Object* copy();
		/*}}}*/
		/*TriaVertexInput management: {{{1*/
		int   EnumType();
		Input* SpawnSingInput(int  index);
		Input* SpawnBeamInput(int* indices);
		Input* SpawnTriaInput(int* indices);
		Input* SpawnProcessedInput(double (*f)(double)){ISSMERROR("not implemented yet");};
		Input* PointwiseDivide(Input* inputB){ISSMERROR("not implemented yet");};
		ElementResult* SpawnResult(int step, double time);
		/*}}}*/
		/*numerics: {{{1*/
		void GetParameterValue(bool* pvalue){ISSMERROR("not implemented yet");}
		void GetParameterValue(int* pvalue){ISSMERROR("not implemented yet");}
		void GetParameterValue(double* pvalue){ISSMERROR("not implemented yet");}
		
		void GetParameterValue(double* pvalue,double* gauss);
		void GetParameterValue(double* pvalue,double* gauss,double defaultvalue){ISSMERROR("not implemented yet");}
		void GetParameterValues(double* values,double* gauss_pointers, int numgauss);
		
		void GetParameterAverage(double* pvalue);
	
		void GetParameterDerivativeValue(double* derivativevalues, double* xyz_list, double* gauss);
		void GetVxStrainRate2d(double* epsilonvx,double* xyz_list, double* gauss);
		void GetVyStrainRate2d(double* epsilonvy,double* xyz_list, double* gauss);
		void GetVxStrainRate3d(double* epsilonvx,double* xyz_list, double* gauss){ISSMERROR("not implemented yet");};
		void GetVyStrainRate3d(double* epsilonvy,double* xyz_list, double* gauss){ISSMERROR("not implemented yet");};
		void GetVzStrainRate3d(double* epsilonvz,double* xyz_list, double* gauss){ISSMERROR("not implemented yet");};
		void GetVxStrainRate3dPattyn(double* epsilonvx,double* xyz_list, double* gauss){ISSMERROR("not implemented yet");};
		void GetVyStrainRate3dPattyn(double* epsilonvy,double* xyz_list, double* gauss){ISSMERROR("not implemented yet");};
		void ChangeEnum(int newenumtype);

		void GetNodalFunctions(double* l1l2l3, double* gauss);
		void GetB(double* B, double* xyz_list, double* gauss);
		void GetNodalFunctionsDerivatives(double* dh1dh3,double* xyz_list, double* gauss);
		void GetNodalFunctionsDerivativesReference(double* dl1dl3,double* gauss_l1l2l3);
		void GetJacobian(double* J, double* xyz_list,double* gauss);
		void GetJacobianInvert(double*  Jinv, double* xyz_list,double* gauss);
		void SquareMin(double* psquaremin, bool process_units,Parameters* parameters);
		void Scale(double scale_factor);
		void AXPY(Input* xinput,double scalar);
		void Constrain(double cm_min, double cm_max);
		void Extrude(void){ISSMERROR("not supported yet");};
		void VerticallyIntegrate(Input* thickness_input){ISSMERROR("not supported yet");};
		void GetVectorFromInputs(Vec vector,int* doflist);
		void GetValuesPtr(double** pvalues,int* pnum_values);
		/*}}}*/

};
#endif  /* _TRIAVERTEXINPUT_H */
